@@ -3,26 +3,16 @@ import json
 
 from django.core.exceptions import ValidationError
 from django.conf import settings
-from django.db import models, DatabaseError, transaction
+from django.db import models
 from django.utils.translation import ugettext_lazy as _
 from django.utils import six
-from django.core.cache import cache
 
-from decimal import Decimal
-import datetime
-
-from .utils import default
+from .utils import _resolve_object_path
 from .widgets import JSONWidget
 from .forms import JSONFormField
-from jsonfield import __version__
-
-DB_TYPE_CACHE_KEY = (
-    'django-jsonfield:db-type:%s' % __version__ +
-    '%(ENGINE)s:%(HOST)s:%(PORT)s:%(NAME)s'
-)
 
 
-class JSONField(six.with_metaclass(models.SubfieldBase, models.Field)):
+class JSONField(models.Field):
     """
     A field that will ensure the data entered into it is valid JSON.
     """
@@ -35,8 +25,15 @@ class JSONField(six.with_metaclass(models.SubfieldBase, models.Field)):
         if not kwargs.get('null', False):
             kwargs['default'] = kwargs.get('default', dict)
         self.encoder_kwargs = {
-            'indent': kwargs.pop('indent', getattr(settings, 'JSONFIELD_INDENT', None))
+            'indent': kwargs.pop('indent', getattr(settings, 'JSONFIELD_INDENT', None)),
         }
+        # This can be an object (probably a class), or a path which can be imported, resulting
+        # in an object.
+        encoder_class = kwargs.pop('encoder_class', getattr(settings, 'JSONFIELD_ENCODER_CLASS', None))
+        if encoder_class:
+            self.encoder_kwargs['cls'] = _resolve_object_path(encoder_class)
+
+        self.decoder_kwargs = dict(kwargs.pop('decoder_kwargs', getattr(settings, 'JSONFIELD_DECODER_KWARGS', {})))
         super(JSONField, self).__init__(*args, **kwargs)
         self.validate(self.get_default(), None)
 
@@ -62,8 +59,8 @@ class JSONField(six.with_metaclass(models.SubfieldBase, models.Field)):
             if callable(default):
                 default = default()
             if isinstance(default, six.string_types):
-                return json.loads(default)
-            return json.loads(json.dumps(default))
+                return json.loads(default, **self.decoder_kwargs)
+            return json.loads(json.dumps(default, **self.encoder_kwargs), **self.decoder_kwargs)
         return super(JSONField, self).get_default()
 
     def get_internal_type(self):
@@ -77,22 +74,15 @@ class JSONField(six.with_metaclass(models.SubfieldBase, models.Field)):
             return 'text'
         if connection.vendor == 'mysql':
             return 'longtext'
+        if connection.vendor == 'oracle':
+            return 'long'
         return 'text'
 
-    def to_python(self, value):
-        if isinstance(value, six.string_types):
-            if value == "":
-                if self.null:
-                    return None
-                if self.blank:
-                    return ""
-            try:
-                value = json.loads(value)
-            except ValueError:
-                msg = self.error_messages['invalid'] % value
-                raise ValidationError(msg)
-        # TODO: Look for date/time/datetime objects within the structure?
-        return value
+    def from_db_value(self, value, expression, connection, context):
+        if value is None:
+            return None
+        return json.loads(value, **self.decoder_kwargs)
+
 
     def get_db_prep_value(self, value, connection=None, prepared=None):
         return self.get_prep_value(value)
@@ -102,14 +92,10 @@ class JSONField(six.with_metaclass(models.SubfieldBase, models.Field)):
             if not self.null and self.blank:
                 return ""
             return None
-        return json.dumps(value, default=default, **self.encoder_kwargs)
+        return json.dumps(value, **self.encoder_kwargs)
 
     def get_prep_lookup(self, lookup_type, value):
-        if lookup_type in ["exact", "iexact"]:
-            return self.to_python(self.get_prep_value(value))
-        if lookup_type == "in":
-            return [self.to_python(self.get_prep_value(v)) for v in value]
-        if lookup_type == "isnull":
+        if lookup_type in ["exact", "iexact", "in", "isnull"]:
             return value
         if lookup_type in ["contains", "icontains"]:
             if isinstance(value, (list, tuple)):
@@ -120,7 +106,7 @@ class JSONField(six.with_metaclass(models.SubfieldBase, models.Field)):
                 return self.get_prep_value(value)[1:-1].replace(', ', r'%')
             if isinstance(value, dict):
                 return self.get_prep_value(value)[1:-1]
-            return self.to_python(self.get_prep_value(value))
+            return self.get_prep_value(value)
         raise TypeError('Lookup type %r not supported' % lookup_type)
 
     def value_to_string(self, obj):
@@ -164,9 +150,3 @@ class TypedJSONField(JSONField):
             else:
                 v(value)
 
-try:
-    from south.modelsinspector import add_introspection_rules
-    add_introspection_rules([], ['^jsonfield\.fields\.JSONField'])
-    add_introspection_rules([], ['^jsonfield\.fields\.TypedJSONField'])
-except ImportError:
-    pass
